﻿//-----------------------------------------------------------------------------
// <copyright file="CRC16.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeDemoApp
{
    /// <summary>
    /// Represents a checksum for the device.
    /// </summary>
    public class CRC16
    {
        /// <summary>
        /// Default CRC16 Prüfpolynom.
        /// </summary>
        private const ushort Polynom = 0x8005;

        /// <summary>
        /// Default Initial CRC16 Value
        /// </summary>
        private const ushort InitialValue = 0xffff;

        /// <summary>
        /// Current CRC16 Prüfpolynom
        /// </summary>
        private ushort polynom = 0x8005;

        /// <summary>
        /// Current CRC16 Value
        /// </summary>
        private ulong crcValue = 0xffff;

        /// <summary>
        /// Initializes a new instance of the CRC16 class
        /// </summary>
        public CRC16()
        {
            this.CRC16_init(CRC16.InitialValue, CRC16.Polynom);
        }

        /// <summary>
        /// Gets the high byte of the crc16 value
        /// </summary>
        public byte CRC16_High
        {
            get
            {
                return (byte)((this.crcValue >> 8) & 0xff);
            }
        }

        /// <summary>
        /// Gets the low byte of the crc16 value
        /// </summary>        
        public byte CRC16_Low
        {
            get
            {
                return (byte)(this.crcValue & 0xff);
            }
        }

        /// <summary>
        /// Gets the bytes of the crc16 value
        /// </summary>        
        public byte[] CrcBytes
        {
            get
            {
                return new byte[] { (byte)((this.crcValue >> 8) & 0xff), (byte)(this.crcValue & 0xff) };
            }
        }

        /// <summary>
        /// Gets the crc16 value
        /// </summary>        
        public ushort Value
        {
            get
            {
                return (ushort)this.crcValue;
            }
        }

        /// <summary>
        /// Initializes the crc16
        /// </summary>
        /// <param name="startValue">Initial value</param>
        /// <param name="polynom">Crc16 polynom</param>
        public void CRC16_init(ushort startValue, ushort polynom)
        {
            this.crcValue = startValue;
            this.polynom = polynom;
        }

        /// <summary>
        /// Process one Message Byte to update the current CRC Value
        /// </summary>
        /// <param name="val">The value for updating the crc value</param>
        public void Shift(int val)
        {
            for (int i = 0; i < 8; i++)
            {
                if (((this.crcValue & 0x8000) ^ (ulong)((val & 0x80) << 8)) == 0x8000)
                {
                    this.crcValue = (this.crcValue << 1) ^ this.polynom;
                }
                else
                {
                    this.crcValue <<= 1;
                }

                val <<= 1;
            }
        }

        /// <summary>
        /// Interates through a array of values
        /// </summary>
        /// <param name="values">Array of values</param>
        public void Shift(byte[] values)
        {
            for (int i = 0; i < values.Length; i++)
            {
                this.Shift(values[i]);
            }
        }
    }
}