﻿//-----------------------------------------------------------------------------
// <copyright file="LedCoordinate.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2012 eQ-3 Entwicklung GmbH
// </copyright>
//-----------------------------------------------------------------------------
namespace RgbCubeDemoApp
{
    using System;

    /// <summary>
    /// Current position of the LED-Snake
    /// </summary>
    public class LedCoordinate
    {
        /// <summary>
        /// Member for X-Property
        /// </summary>
        private int x;

        /// <summary>
        /// Member for Y-Property
        /// </summary>
        private int y;

        /// <summary>
        /// Member for Z-Property
        /// </summary>
        private int z;

        /// <summary>
        /// Red-Value of RGB.
        /// </summary>
        private int r;

        /// <summary>
        /// Green-Value of RGB.
        /// </summary>
        private int g;

        /// <summary>
        /// Blue-Value of RGB.
        /// </summary>
        private int b;

        /// <summary>
        /// Initializes a new instance of the LedCoordinate class
        /// </summary>
        public LedCoordinate()
        {
            this.X = 0;
            this.Y = 0;
            this.Z = 0;
            this.R = 0;
            this.G = 0;
            this.B = 0;
        }

        /// <summary>
        /// Initializes a new instance of the LedCoordinate class
        /// </summary>
        /// <param name="x">X-coordinate of the LED</param>
        /// <param name="y">Y-coordinate of the LED</param>
        /// <param name="z">Z-coordinate of the LED</param>
        public LedCoordinate(int x, int y, int z)
        {
            this.X = x;
            this.Y = y;
            this.Z = z;
            this.R = 0;
            this.G = 0;
            this.B = 0;
        }

        ///// <summary>
        ///// Initializes a new instance of the LedCoordinate class
        ///// </summary>
        ///// <param name="x">X-coordinate of the LED</param>
        ///// <param name="y">Y-coordinate of the LED</param>
        ///// <param name="z">Z-coordinate of the LED</param>

        /// <summary>
        /// Initializes a new instance of the LedCoordinate class
        /// </summary>
        /// <param name="x">X-coordinate of the LED</param>
        /// <param name="y">Y-coordinate of the LED</param>
        /// <param name="z">Z-coordinate of the LED</param>
        /// <param name="r">Red-Value of RGB</param>
        /// <param name="g">Green-Value of RGB</param>
        /// <param name="b">Blue-Value of RGB</param>
        public LedCoordinate(int x, int y, int z, int r, int g, int b)
        {
            this.X = x;
            this.Y = y;
            this.Z = z;
            this.R = r;
            this.G = g;
            this.B = b;
        }

        /// <summary>
        /// Gets or sets the X-coordinate of the LED
        /// </summary>
        public int X
        {
            get
            {
                return this.x;
            }

            set
            {
                if (value > 4)
                {
                    this.x = 0;
                }
                else
                {
                    this.x = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets the Y-coordinate of the LED
        /// </summary>
        public int Y
        {
            get
            {
                return this.y;
            }

            set
            {
                if (value > 4)
                {
                    this.y = 0;
                }
                else
                {
                    this.y = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets the Z-coordinate of the LED
        /// </summary>
        public int Z
        {
            get
            {
                return this.z;
            }

            set
            {
                if (value > 4)
                {
                    this.z = 0;
                }
                else
                {
                    this.z = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets red value of RGB.
        /// </summary>
        public int R
        {
            get
            {
                return this.r;
            }

            set
            {
                this.r = value;
            }
        }

        /// <summary>
        /// Gets or sets green value of RGB.
        /// </summary>
        public int G
        {
            get
            {
                return this.g;
            }

            set
            {
                this.g = value;
            }
        }

        /// <summary>
        /// Gets or sets blue value of RGB.
        /// </summary>
        public int B
        {
            get
            {
                return this.b;
            }

            set
            {
                this.b = value;
            }
        }

        /// <summary>
        /// Move the position of the LED-Snake. By chance the x-, y- or z-coordinate gets incremented
        /// </summary>
        /// <returns>A new instance of the LedCoordinate class with the new coordinates</returns>
        public LedCoordinate MovePoint()
        {
            Random random = new Random();
            int choice;

            choice = random.Next(3);

            switch (choice)
            {
                case 0:
                    this.X++;
                    break;
                case 1:
                    this.Y++;
                    break;
                case 2:
                    this.Z++;
                    break;
            }

            return new LedCoordinate(this.X, this.Y, this.Z);
        }
    }
}