﻿namespace RgbCubeSpectrumAnalyzer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows;
    using System.Collections.ObjectModel;

    /// <summary>
    /// DependencyProperties of the MainWindow class.
    /// </summary>
    public partial class MainWindow
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for TogglePlayerButtonText.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty TogglePlayerButtonTextProperty =
            DependencyProperty.Register("TogglePlayerButtonText", typeof(string), typeof(MainWindow), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ComPortNumber.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ComPortNumberProperty =
            DependencyProperty.Register("ComPortNumber", typeof(string), typeof(MainWindow), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ConnectedToComPort.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ConnectedToComPortProperty =
            DependencyProperty.Register("ConnectedToComPort", typeof(bool), typeof(MainWindow), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ConnectButtonText.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ConnectButtonTextProperty =
            DependencyProperty.Register("ConnectButtonText", typeof(string), typeof(MainWindow), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentMediaFile.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentMediaFileProperty =
            DependencyProperty.Register("CurrentMediaFile", typeof(string), typeof(MainWindow), new UIPropertyMetadata(MainWindow.CurrentMediaFileNoFile));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ComPortConnectionEditable.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ComPortConnectionEditableProperty =
            DependencyProperty.Register("ComPortConnectionEditable", typeof(bool), typeof(MainWindow), new UIPropertyMetadata(true));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentTrackTime.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentTrackTimeProperty =
            DependencyProperty.Register("CurrentTrackTime", typeof(string), typeof(MainWindow), new UIPropertyMetadata(MainWindow.DefaultTrackTime));

        /// <summary>
        /// Using a DependencyProperty as the backing store for TotalTrackTime.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty TotalTrackTimeProperty =
            DependencyProperty.Register("TotalTrackTime", typeof(string), typeof(MainWindow), new UIPropertyMetadata(MainWindow.DefaultTrackTime));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AvailableComports.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AvailableComportsProperty =
            DependencyProperty.Register("AvailableComports", typeof(ObservableCollection<string>), typeof(MainWindow), new UIPropertyMetadata(null));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Tracklist.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty TracklistProperty =
            DependencyProperty.Register("Tracklist", typeof(ObservableCollection<string>), typeof(MainWindow), new UIPropertyMetadata(null));

        /// <summary>
        /// Text of the play-button to play the audio track.
        /// </summary>
        private const string PlayerButtonTextPlay = "4";

        /// <summary>
        /// Text of the play-button to pause the playback.
        /// </summary>
        private const string PlayerButtonTextPause = ";";

        /// <summary>
        /// Text of the connect-button to connect with selected ComPort.
        /// </summary>
        private const string ConnectButtonConnect = "Verbindung mit ComPort aufbauen";

        /// <summary>
        /// Text of the connect-button to disconnect from ComPort.
        /// </summary>
        private const string ConnectButtonDisconnect = "ComPort-Verbindung trennen";

        /// <summary>
        /// Text, which appears when there is no media file selected.
        /// </summary>
        private const string CurrentMediaFileNoFile = "[Wiedergabe gestoppt - keine Datei ausgewählt]";
        
        /// <summary>
        /// Text of the default track time.
        /// </summary>
        private const string DefaultTrackTime = "00:00:00";
        
        /// <summary>
        /// Gets or sets the text of the play-button.
        /// </summary>
        public string TogglePlayerButtonText
        {
            get { return (string)GetValue(TogglePlayerButtonTextProperty); }
            set { SetValue(TogglePlayerButtonTextProperty, value); }
        }

        /// <summary>
        /// Gets or sets the number of the ComPort.
        /// </summary>
        public string ComPortNumber
        {
            get { return (string)GetValue(ComPortNumberProperty); }
            set { SetValue(ComPortNumberProperty, value); }
        }

        /// <summary>
        /// Gets or sets the connection to the ComPort.
        /// </summary>
        public bool ConnectedToComPort
        {
            get { return (bool)GetValue(ConnectedToComPortProperty); }
            set { SetValue(ConnectedToComPortProperty, value); }
        }

        /// <summary>
        /// Gets or sets the text of the connect-button.
        /// </summary>
        public string ConnectButtonText
        {
            get { return (string)GetValue(ConnectButtonTextProperty); }
            set { SetValue(ConnectButtonTextProperty, value); }
        }

        /// <summary>
        /// Gets or sets the current media file.
        /// </summary>
        public string CurrentMediaFile
        {
            get { return (string)GetValue(CurrentMediaFileProperty); }
            set { SetValue(CurrentMediaFileProperty, value); }
        }

        /// <summary>
        /// Gets or sets the editability of the connection to the ComPort.
        /// </summary>
        public bool ComPortConnectionEditable
        {
            get { return (bool)GetValue(ComPortConnectionEditableProperty); }
            set { SetValue(ComPortConnectionEditableProperty, value); }
        }

        /// <summary>
        /// Gets or sets the current track time of the audio playback.
        /// </summary>
        public string CurrentTrackTime
        {
            get { return (string)GetValue(CurrentTrackTimeProperty); }
            set { SetValue(CurrentTrackTimeProperty, value); }
        }

        /// <summary>
        /// Gets or sets the total track time of the audio playback.
        /// </summary>
        public string TotalTrackTime
        {
            get { return (string)GetValue(TotalTrackTimeProperty); }
            set { SetValue(TotalTrackTimeProperty, value); }
        }

        /// <summary>
        /// Gets or sets the aviable ComPorts.
        /// </summary>
        public ObservableCollection<string> AvailableComports
        {
            get { return (ObservableCollection<string>)GetValue(AvailableComportsProperty); }
            set { SetValue(AvailableComportsProperty, value); }
        }

        /// <summary>
        /// Gets or sets the playlist.
        /// </summary>
        public ObservableCollection<string> Tracklist
        {
            get { return (ObservableCollection<string>)GetValue(TracklistProperty); }
            set { SetValue(TracklistProperty, value); }
        }
    }
}
